"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Input = exports.Form = void 0;
const jsx_runtime_1 = require("react/jsx-runtime");
const react_1 = require("react");
const ink_1 = require("ink");
const ink_2 = require("ink");
const utils_1 = require("../utils");
const Form = ({ inputs: inputDescriptors, onExit, onSubmit, }) => {
    const initialInputs = inputDescriptors.map((input) => { var _a; return (_a = input.initialValue) !== null && _a !== void 0 ? _a : ''; });
    const [state, setState] = react_1.useState({
        inputs: initialInputs,
        cursorOffset: 0,
        activeInputIndex: 1,
    });
    const { inputs, cursorOffset, activeInputIndex } = state;
    const activeInputValue = inputs[activeInputIndex];
    ink_1.useInput((input, key) => {
        if (key.escape || (key.ctrl && (input === 'c' || input === 'd'))) {
            onExit();
            return;
        }
        else if (key.ctrl && input === 'u') {
            setState({
                ...state,
                inputs: utils_1.replaceElement(inputs, activeInputIndex, ''),
                cursorOffset: 0,
            });
        }
        else if (key.ctrl && input === 'a') {
            setState({ ...state, cursorOffset: 0 });
        }
        else if (key.ctrl && input === 'e') {
            setState({ ...state, cursorOffset: activeInputValue.length });
        }
        else if (key.leftArrow) {
            if (cursorOffset > 0) {
                setState({ ...state, cursorOffset: cursorOffset - 1 });
            }
        }
        else if (key.rightArrow) {
            if (cursorOffset < activeInputValue.length) {
                setState({ ...state, cursorOffset: cursorOffset + 1 });
            }
        }
        else if (key.return) {
            onSubmit(state);
            return;
        }
        else if (key.upArrow || (key.tab && key.shift)) {
            const newInputIndex = (activeInputIndex + 3 - 1) % 3;
            const newInputValue = inputs[newInputIndex];
            setState({
                ...state,
                activeInputIndex: newInputIndex,
                cursorOffset: key.tab
                    ? 0
                    : Math.min(cursorOffset, newInputValue.length),
            });
        }
        else if (key.downArrow || key.tab) {
            const newInputIndex = (activeInputIndex + 3 + 1) % 3;
            const newInputValue = inputs[newInputIndex];
            setState({
                ...state,
                activeInputIndex: newInputIndex,
                cursorOffset: key.tab
                    ? 0
                    : Math.min(cursorOffset, newInputValue.length),
            });
        }
        else {
            if (key.delete || key.backspace) {
                if (cursorOffset > 0) {
                    const newInputValue = activeInputValue.slice(0, cursorOffset - 1) +
                        activeInputValue.slice(cursorOffset);
                    setState({
                        ...state,
                        cursorOffset: cursorOffset - 1,
                        inputs: utils_1.replaceElement(inputs, activeInputIndex, newInputValue),
                    });
                }
            }
            else {
                const newInputValue = activeInputValue.slice(0, cursorOffset) +
                    input +
                    activeInputValue.slice(cursorOffset);
                setState({
                    ...state,
                    cursorOffset: cursorOffset + input.length,
                    inputs: utils_1.replaceElement(inputs, activeInputIndex, newInputValue),
                });
            }
        }
    });
    return (jsx_runtime_1.jsx(ink_2.Box, Object.assign({ flexDirection: "column" }, { children: inputDescriptors.map((descriptor, index) => {
            return (jsx_runtime_1.jsx(exports.Input, { placeholder: descriptor.placeholder, initialValue: descriptor.initialValue, onChange: descriptor.onChange, label: descriptor.label, validationError: descriptor.validationError, active: activeInputIndex === index, value: state.inputs[index], cursorOffset: cursorOffset }, index));
        }) }), void 0));
};
exports.Form = Form;
const Input = ({ placeholder, active, value, validationError, onChange, cursorOffset, label, }) => {
    react_1.useEffect(() => onChange(value), [value]);
    const textWithCursor = (text) => {
        if (!text) {
            return jsx_runtime_1.jsx(ink_2.Text, Object.assign({ inverse: true }, { children: " " }), void 0);
        }
        if (cursorOffset === text.length) {
            return (jsx_runtime_1.jsxs(ink_2.Text, { children: [text, jsx_runtime_1.jsx(ink_2.Text, Object.assign({ inverse: true }, { children: " " }), void 0)] }, void 0));
        }
        return (jsx_runtime_1.jsxs(jsx_runtime_1.Fragment, { children: [jsx_runtime_1.jsx(ink_2.Text, { children: text.substr(0, cursorOffset) }, void 0),
                jsx_runtime_1.jsx(ink_2.Text, Object.assign({ inverse: true }, { children: text.substr(cursorOffset, 1) }), void 0),
                jsx_runtime_1.jsx(ink_2.Text, { children: text.substr(cursorOffset + 1) }, void 0)] }, void 0));
    };
    return (jsx_runtime_1.jsxs(ink_2.Text, { children: [jsx_runtime_1.jsx(ink_2.Text, Object.assign({ color: "blue", dimColor: !active }, { children: label }), void 0),
            jsx_runtime_1.jsx(ink_2.Text, Object.assign({ dimColor: !active }, { children: '|' }), void 0),
            !value && placeholder ? (jsx_runtime_1.jsx(ink_2.Text, Object.assign({ color: "dim" }, { children: active ? textWithCursor(placeholder) : placeholder }), void 0)) : (jsx_runtime_1.jsxs(ink_2.Text, { children: [active ? textWithCursor(value) : value, jsx_runtime_1.jsxs(ink_2.Text, Object.assign({ color: "red" }, { children: [" ", validationError] }), void 0)] }, void 0))] }, void 0));
};
exports.Input = Input;
