"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.interactive = void 0;
const jsx_runtime_1 = require("react/jsx-runtime");
const react_1 = require("react");
const ink_1 = require("ink");
const getFiles_1 = require("../getFiles");
const Inputs_1 = require("./Inputs");
const inputs = ['pattern', 'search', 'replace'];
const interactive = async ({ initialPattern, initialSearch, initialReplace, }) => {
    return new Promise((resolve, reject) => {
        let clear;
        const App = () => {
            const { exit } = ink_1.useApp();
            const [state, setState] = react_1.useState({
                pattern: initialPattern || '**',
                search: initialSearch || '',
                replace: initialReplace || '',
                cursorOffset: 0,
                activeInputIndex: 1,
            });
            react_1.useEffect(() => {
                if (pattern) {
                    getFiles_1.getFiles(pattern).then((f) => setFiles(f));
                }
            }, []);
            const [files, setFiles] = react_1.useState([]);
            const { pattern, search, replace, cursorOffset, activeInputIndex, } = state;
            const activeInputName = inputs[activeInputIndex];
            const activeInputValue = state[activeInputName];
            ink_1.useInput((input, key) => {
                if (key.escape || (key.ctrl && (input === 'c' || input === 'd'))) {
                    clear();
                    exit();
                    reject('abort');
                    return;
                }
                else if (key.ctrl && input === 'u') {
                    setState({ ...state, [activeInputName]: '', cursorOffset: 0 });
                }
                else if (key.ctrl && input === 'a') {
                    setState({ ...state, cursorOffset: 0 });
                }
                else if (key.ctrl && input === 'e') {
                    setState({ ...state, cursorOffset: activeInputValue.length });
                }
                else if (key.leftArrow) {
                    if (cursorOffset > 0) {
                        setState({ ...state, cursorOffset: cursorOffset - 1 });
                    }
                }
                else if (key.rightArrow) {
                    if (cursorOffset < activeInputValue.length) {
                        setState({ ...state, cursorOffset: cursorOffset + 1 });
                    }
                }
                else if (key.return) {
                    clear();
                    exit();
                    resolve({ pattern, search, replace });
                    return;
                }
                else if (key.upArrow || (key.tab && key.shift)) {
                    const newInputIndex = (activeInputIndex + 3 - 1) % 3;
                    const newInputName = inputs[newInputIndex];
                    const newInputValue = state[newInputName];
                    setState({
                        ...state,
                        activeInputIndex: newInputIndex,
                        cursorOffset: key.tab
                            ? 0
                            : Math.min(cursorOffset, newInputValue.length),
                    });
                }
                else if (key.downArrow || key.tab) {
                    const newInputIndex = (activeInputIndex + 3 + 1) % 3;
                    const newInputName = inputs[newInputIndex];
                    const newInputValue = state[newInputName];
                    setState({
                        ...state,
                        activeInputIndex: newInputIndex,
                        cursorOffset: key.tab
                            ? 0
                            : Math.min(cursorOffset, newInputValue.length),
                    });
                }
                else {
                    if (key.delete || key.backspace) {
                        if (cursorOffset > 0) {
                            const newInputValue = activeInputValue.slice(0, cursorOffset - 1) +
                                activeInputValue.slice(cursorOffset);
                            if (activeInputName === 'pattern') {
                                if (!newInputValue) {
                                    setFiles([]);
                                }
                                else {
                                    getFiles_1.getFiles(newInputValue).then(setFiles);
                                }
                            }
                            setState({
                                ...state,
                                cursorOffset: cursorOffset - 1,
                                [activeInputName]: newInputValue,
                            });
                        }
                    }
                    else {
                        const newInputValue = activeInputValue.slice(0, cursorOffset) +
                            input +
                            activeInputValue.slice(cursorOffset);
                        if (activeInputName === 'pattern') {
                            if (!newInputValue) {
                                setFiles([]);
                            }
                            else {
                                getFiles_1.getFiles(newInputValue).then((f) => setFiles(f));
                            }
                        }
                        setState({
                            ...state,
                            cursorOffset: cursorOffset + input.length,
                            [activeInputName]: newInputValue,
                        });
                    }
                }
            });
            let searchRegexp;
            let searchError = '';
            try {
                searchRegexp = new RegExp(search);
            }
            catch (error) {
                searchError = error.message;
            }
            return (jsx_runtime_1.jsxs(jsx_runtime_1.Fragment, { children: [jsx_runtime_1.jsx(Inputs_1.Inputs, { pattern: pattern, search: search, replace: replace, activeInputIndex: activeInputIndex, cursorOffset: cursorOffset, searchError: searchError }, void 0),
                    jsx_runtime_1.jsx(Monitor, { files: files, searchRegexp: searchRegexp, replace: replace }, void 0)] }, void 0));
        };
        const Monitor = ({ files, searchRegexp, replace, }) => {
            if (files.length === 0) {
                return jsx_runtime_1.jsx(jsx_runtime_1.Fragment, {}, void 0);
            }
            return (jsx_runtime_1.jsx(jsx_runtime_1.Fragment, { children: files
                    .map((filePath) => {
                    if (!searchRegexp) {
                        return jsx_runtime_1.jsx(ink_1.Text, { children: filePath }, filePath);
                    }
                    let result = searchRegexp.exec(filePath);
                    if (result === null) {
                        return jsx_runtime_1.jsx(ink_1.Text, { children: filePath }, filePath);
                    }
                    const match = result[0];
                    const matchStartIndex = result.index;
                    return (jsx_runtime_1.jsxs(ink_1.Text, { children: [jsx_runtime_1.jsx(ink_1.Text, { children: filePath.substr(0, matchStartIndex) }, void 0),
                            jsx_runtime_1.jsx(ink_1.Text, Object.assign({ strikethrough: true, color: "red" }, { children: match }), void 0),
                            jsx_runtime_1.jsx(ink_1.Text, Object.assign({ color: "green" }, { children: replace }), void 0),
                            jsx_runtime_1.jsx(ink_1.Text, { children: filePath.substr(matchStartIndex + match.length) }, void 0)] }, filePath));
                })
                    .slice(0, process.stdout.rows - 5) }, void 0));
        };
        const inkRenderApi = ink_1.render(jsx_runtime_1.jsx(App, {}, void 0), { exitOnCtrlC: false });
        clear = inkRenderApi.clear;
    });
};
exports.interactive = interactive;
